library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity memoryctrl is
    Port ( rst : in std_logic;
           clk : in std_logic;
           cs : in std_logic;
           select0 : in std_logic;
           rnw : in std_logic;
           vreq : in std_logic;
           onecycle : in std_logic;
           videocycle : out std_logic;
           hihalf : out std_logic;
           pb_wr : out std_logic;
           pb_rd : out std_logic;
           xfer : out std_logic;
           ce0 : out std_logic;
           ce1 : out std_logic;
           rres : out std_logic;
           video_ce : out std_logic
			  );
end memoryctrl;

architecture Behavioral of memoryctrl is

signal r_idle: std_logic;
signal r_common: std_logic;
signal r_ra1: std_logic;
signal r_ra2: std_logic;
signal r_rb1: std_logic;
signal r_rb2: std_logic;
signal r_rb3: std_logic;
signal r_w: std_logic;
signal r_xfer: std_logic;
signal vcycle_1: std_logic;
signal vcycle_2: std_logic;
signal videocycle_temp: std_logic;

begin

process (rst, clk, cs, select0, rnw, vreq, onecycle )
begin

	if rst = '1' then
		r_idle <= '1'; r_common <= '0';
		r_ra1 <= '0'; r_ra2 <= '0'; r_rb1 <= '0'; r_rb2 <= '0'; r_rb3 <= '0';
		r_w <= '0';
		r_xfer <= '0';

	elsif CLK'event and CLK='1' then		-- positive clk edge
		r_idle <= (r_idle and (not cs)) or r_xfer or (not select0);
		r_common <= select0 and ((r_idle and cs) or (r_common and vreq));
		r_ra1 <= select0 and r_common and (not vreq) and rnw;
		r_ra2 <= select0 and r_ra1;
		r_rb1 <= select0 and ((r_common and not onecycle and not vreq and rnw) or (r_rb1 and vreq));
		r_rb2 <= select0 and (r_rb1 and (not vreq));
		r_rb3 <= select0 and r_rb2;
		r_w <= select0 and ((r_common and (not rnw) and (not vreq) and (not onecycle)) or (r_w and vreq));


               r_xfer <= select0 and ((r_common and onecycle and (not rnw) and (not vreq)) or (r_w and (not vreq)) or (r_ra2 and onecycle) or r_rb3);
--                r_xfer <= select0 and (r_common and not vreq); 

                
	end if;

	pb_wr <= (r_common and (not rnw) and (not vreq)) or (r_w and (not vreq));		
	pb_rd <= vreq or (r_common and (not vreq) and rnw) or (r_rb1 and (not vreq));
	hihalf <= (r_w and (not vreq)) or (r_rb1 and (not vreq));
	videocycle <= vreq;
	videocycle_temp <=vreq;

	if(rst = '1') then
		vcycle_1 <= '0';
		vcycle_2 <= '0';
	elsif CLK'event and CLK='1' then	
		vcycle_1 <= videocycle_temp;
		vcycle_2 <= vcycle_1;
	end if;
	
	video_ce <= vcycle_2;
	ce0 <= r_ra2;
	ce1 <= r_rb3;
	rres <= r_xfer;
	xfer <= r_xfer;

end process;
		
end Behavioral;
