library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library UNISIM;		
use UNISIM.VCOMPONENTS.ALL;

entity pad_io is
    Port ( clk : in std_logic;
           rst : in std_logic;
           PB_A : out std_logic_vector(19 downto 0);
           PB_UB_N : out std_logic;
           PB_LB_N : out std_logic;
           PB_WE_N : out std_logic;
           PB_OE_N : out std_logic;
           RAM_CE_N : out std_logic;
           PB_D : inout std_logic_vector(15 downto 0);
           pb_addr : in std_logic_vector(19 downto 0);
           pb_ub : in std_logic;
           pb_lb : in std_logic;
           pb_wr : in std_logic;
           pb_rd : in std_logic;
           ram_ce : in std_logic;
           pb_dread : out std_logic_vector(15 downto 0);
           pb_dwrite : in std_logic_vector(15 downto 0));
end pad_io;

architecture Behavioral of pad_io is

component FDC
port (C : in std_logic;
		CLR : in std_logic;
		D : in std_logic;
		Q : out std_logic);
end component;

component FDP
port (C : in std_logic;
		PRE : in std_logic;
		D : in std_logic;
		Q : out std_logic);
end component;

attribute iob : string;
attribute iob of FDC : component is "true";
attribute iob of FDP : component is "true";

component OBUF_F_24
port (O : out STD_ULOGIC;
		I : in STD_ULOGIC);
end component;

component IOBUF_F_24
port (O : out STD_ULOGIC;
		IO : inout STD_ULOGIC;
		I : in STD_ULOGIC;
		T : in STD_ULOGIC);
end component;

signal pb_addr_1: std_logic_vector(19 downto 0);
signal pb_dwrite_1: std_logic_vector(15 downto 0);
signal pb_tristate: std_logic_vector(15 downto 0);
signal pb_tristate_1: std_logic_vector(15 downto 0);
signal pb_dread_a: std_logic_vector(15 downto 0);
signal we_n, pb_we_n1: std_logic;
signal oe_n, pb_oe_n1: std_logic;
signal lb_n, pb_lb_n1: std_logic;
signal ub_n, pb_ub_n1: std_logic;
signal ramce_n, ram_ce_n1: std_logic;
signal dataz : std_logic;	

--attribute equivalent_register_removal: string;
--attribute equivalent_register_removal of pb_tristate_1 : signal is "no";
--attribute equivalent_register_removal of pb_dwrite_1 : signal is "no";

begin

dataz <= (not pb_wr) or pb_rd;

process (dataz)
begin
if dataz = '1' then pb_tristate <= "1111111111111111"; else pb_tristate <= "0000000000000000"; end if;
end process;

webuf : OBUF_F_24
port map (O => PB_WE_N,
			 I => pb_we_n1);

oebuf : OBUF_F_24
port map (O => PB_OE_N,
			 I => pb_oe_n1);

ramcebuf : OBUF_F_24
port map (O => RAM_CE_N,
			 I => ram_ce_n1);

ubbuf : OBUF_F_24
port map (O => PB_UB_N,
			 I => pb_ub_n1);

lbbuf : OBUF_F_24
port map (O => PB_LB_N,
			 I => pb_lb_n1);

abuf : for i in 0 to 19 generate
	abuf : OBUF_F_24 port map (
		O => PB_A(i),
		I => pb_addr_1(i));
end generate;

dbuf : for i in 0 to 15 generate
	dbuf : IOBUF_F_24 port map (
		O => pb_dread_a(i),
		IO => PB_D(i),
		I => pb_dwrite_1(i),
		T => pb_tristate_1(i));
end generate;

aff : for i in 0 to 19 generate
	aff : FDC port map (
		C => clk, CLR => rst,
		D => pb_addr(i),
		Q => pb_addr_1(i));
end generate;

dff : for i in 0 to 15 generate
	drff : FDP port map (
		C => clk, PRE => rst,
		D => pb_dread_a(i),
		Q => pb_dread(i));

	dwff : FDP port map (
		C => clk, PRE => rst,
		D => pb_dwrite(i),
		Q => pb_dwrite_1(i));

	dtff : FDP port map (
		C => clk, PRE => rst,
		D => pb_tristate(i),
		Q => pb_tristate_1(i));

end generate;

we_n <= not pb_wr;
oe_n <= not pb_rd;
lb_n <= not pb_lb;
ub_n <= not pb_ub;
ramce_n <= not ram_ce;

weff : FDP port map (
		C => clk, PRE => rst,
		D => we_n,
		Q => pb_we_n1);

oeff : FDP port map (
		C => clk, PRE => rst,
		D => oe_n,
		Q => pb_oe_n1);

lbff : FDP port map (
		C => clk, PRE => rst,
		D => lb_n,
		Q => pb_lb_n1);

ubff : FDP port map (
		C => clk, PRE => rst,
		D => ub_n,
		Q => pb_ub_n1);

ramceff : FDP port map (
		C => clk, PRE => rst,
		D => ramce_n,
		Q => ram_ce_n1);

end Behavioral;
